using System;
using UnityEngine;
using UnityEngine.Experimental.Rendering;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Linq;
using MVR;
using MeshVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using GPUTools.Cloth.Scripts;
using Unity.Collections;
using AssetBundles;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Stopper {
	public class ClothingDrivenMorphs : MVRScript {
        DAZClothingItem clothingItem;
        DAZCharacterSelector selector;
        DAZMorph selectedMorph;

        // UI Stuff
        MVRScriptExtensions.VerticalLayoutPanel mainPanel;
        MVRScriptExtensions.VerticalLayoutPanel selectionPanel;
        MVRScriptExtensions.VerticalLayoutPanel morphsPanel;

        List<DAZMorph> controlledMorphs = new List<DAZMorph>();

        Dictionary<DAZMorph, float> initialMorphValues = new Dictionary<DAZMorph, float>();

        JSONStorableBool ignoreIfNaturalisPresent;
        bool naturalisPresent = false;

		public override void Init() {
			WalkAndGetParentItems();

            selector = containingAtom.GetStorableByID("geometry") as DAZCharacterSelector;
            if(selector == null) {
                throw new Exception("Unable to find character selector. Is this a person atom?");
            }

            MVRPluginManager pluginManager = containingAtom.GetStorableByID("PluginManager") as MVRPluginManager;
            foreach(MVRScript script in pluginManager.GetComponentsInChildren<MVRScript>(true)) {
                if(script.GetType().ToString().ToLower().Contains("naturalis")) {
                    naturalisPresent = true;
                }
            }
		}

        public void InitMainUI() {
            if(mainPanel == null) {
                var mainUI = this.rightUIContent.transform.parent;
                mainPanel = this.AddUIElement<MVRScriptExtensions.VerticalLayoutPanel>("MainPanel", mainUI);

                morphsPanel = this.AddUIElement<MVRScriptExtensions.VerticalLayoutPanel>("MorphsPanel", mainPanel);
                selectionPanel = this.AddUIElement<MVRScriptExtensions.VerticalLayoutPanel>("SelectionPanel", mainPanel);
                selectionPanel.layoutElement.ignoreLayout = true;

                var spacerPanel = this.AddUIElement<MVRScriptExtensions.MaxSizeLayoutPanel>("SpacerPanel", morphsPanel);
                
                JSONStorableStringChooser morphChooser = new JSONStorableStringChooser(
                        "Morph Uid", 
                        selector.morphsControlUI.GetMorphUids(),
                        null, 
                        "Morph Uid", 
                        (string uid) => {
                            selectedMorph = selector.morphsControlUI.GetMorphByUid(uid);
                        });
                        
                RegisterStringChooser(morphChooser);
                var selectMorphPopup = CreateFilterablePopup(morphChooser, false);
                selectMorphPopup.popup.onOpenPopupHandlers += () => {
                    morphChooser.choices = selector.morphsControlUI.GetMorphUids();
                };

                this.AddToPanel(selectMorphPopup, selectionPanel);
       
                var addMorphButton = CreateButton("Add Morph");
                addMorphButton.button.onClick.AddListener(() => {
                    AddMorph(selectedMorph);
                });
                this.AddToPanel(addMorphButton, selectionPanel);
                addMorphButton.height = 70;

                ignoreIfNaturalisPresent = new JSONStorableBool("IgnoreIfNaturalisPresent", false);
                var ignoreNaturalisToggle = this.CreateToggle(ignoreIfNaturalisPresent);
                this.AddToPanel(ignoreNaturalisToggle, selectionPanel);
                ignoreNaturalisToggle.label = "Ignore If Naturalis Present";

                spacerPanel.layoutElement.minHeight = 260;
                spacerPanel.layoutElement.preferredHeight = 260;
            }
        }

        public void Start() {
            InitMainUI();
        }
		
        private void WalkAndGetParentItems(){
			GameObject parent = this.gameObject;
			while(parent != null && parent != containingAtom.gameObject){
				if(parent.transform.parent == null){
					return;
				}
				
				if(clothingItem == null){
					clothingItem = parent.GetComponent<DAZClothingItem>();
				}

				parent = parent.transform.parent.gameObject;
			}
		}

        private void AddMorph(DAZMorph morph) {
            if(morph == null || controlledMorphs.Contains(morph)) {
                return;
            }
            initialMorphValues[morph] = morph.morphValue;
            controlledMorphs.Add(morph);
            CreateMorphPanel(morph);
        }

        private void CreateMorphPanel(DAZMorph morph) {
            var morphPanel = this.AddUIElement<MVRScriptExtensions.VerticalSplitLayoutPanel>(morph.uid, morphsPanel);
            morphPanel.ratio = 0.8f;

            string sliderStorableName =  morph.uid.Replace(':', ';');
            var slider = this.AddSlider(sliderStorableName, morphPanel.leftPanel, morph.jsonFloat.defaultVal, morph.min, morph.max, (val)=> {
                SetMorphValue(morph, val);
            });
            slider.label = morph.displayName;
            JSONStorableFloat storableFloat = this.sliderToJSONStorableFloat[slider];

            var removeButton = CreateButton("Remove");
            removeButton.button.onClick.AddListener(() => {
                if(initialMorphValues.ContainsKey(morph)) {
                    SetMorphValue(morph, initialMorphValues[morph]);
                    initialMorphValues.Remove(morph);
                }
                controlledMorphs.Remove(morph);
                DeregisterFloat(storableFloat);
                Destroy(morphPanel.gameObject);
            });
            removeButton.height = slider.height;
            this.AddToPanel(removeButton, morphPanel.rightPanel);
        }

        public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true){
            InitMainUI();
            if(jc.HasKey("controlledMorphs")) {
                JSONArray morphsJson = jc["controlledMorphs"].AsArray;
                foreach(JSONNode morphUid in morphsJson) {
                    AddMorph(selector.morphsControlUI.GetMorphByUid(morphUid.Value));
                }
            }

            base.RestoreFromJSON(jc,restorePhysical,restoreAppearance,presetAtoms,false);
        }

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false) {
            JSONClass json = base.GetJSON(includePhysical, includeAppearance, forceStore);

            JSONArray morphsJson = new JSONArray();
            foreach(DAZMorph morph in controlledMorphs) {
                morphsJson.Add(morph.uid);
            }
            json["controlledMorphs"] = morphsJson;
            return json;
        }

        private void SetMorphValue(DAZMorph morph, float val) {
            if(ignoreIfNaturalisPresent.val && naturalisPresent) {
                return;
            }
            morph.SetValue(val);
            morph.jsonFloat.val = val;
        }

        private void OnEnable() {
            foreach(var entry in initialMorphValues.ToList()) {
                initialMorphValues[entry.Key] = entry.Key.morphValue;
            }

            foreach(var entry in this.floatParams) {
                entry.Value.setCallbackFunction(entry.Value.val);
            }
        }

		private void OnDisable()
		{
			foreach(var entry in initialMorphValues) {
                SetMorphValue(entry.Key, entry.Value);
            }
		}
	}
}